#include "common_porting.h"
#include "cmsis_os.h"
#include "stm32f4xx_hal.h"
#include "bma456h_task.h"
#include "bma456h.h"


#if defined(USE_BMA456H)
extern volatile uint8_t int0_flag;
extern volatile uint8_t int1_flag;

/* Declare an instance of the BMA4xy device */
struct bma4_dev bma456accel;

#if defined(FIFO_POLL) || defined(FIFO_WM_INT) || defined(STEP_COUNTER)
TimerHandle_t BMA456H_Timer_Handler;
volatile uint16_t bma456h_fifo_ready = 0;

/* Declare memory to store the raw FIFO buffer information */   
uint8_t fifo_buff[420];//for 50 Hz ODR
struct bma4_fifo_frame bma456h_fifo_frame;

/* Declare instances of the sensor data structure */
struct bma4_accel bma456h_sensor_data[60]; 
uint16_t bma456h_fifo_length, bma456h_acc_frames;
#endif

#if defined(STEP_COUNTER)
uint32_t bma456step_cnt;
#endif

#if defined(ANY_MOTION)
/* Structure to define any-motion */
struct bma456h_any_no_mot_config any_mot = {0};
#endif

void print_bma456_rslt(int16_t rslt)
{
    switch (rslt)
    {
        case BMA4_OK:
            PDEBUG("Successfully completed\r\n");
            break;
        case BMA4_E_NULL_PTR:
            PDEBUG("Error [%d] : Null pointer\r\n", rslt);
            break;
        case BMA4_E_OUT_OF_RANGE:
            PDEBUG("Error [%d] : Out of range\r\n", rslt);
            break;
        case BMA4_E_INVALID_SENSOR:
            PDEBUG("Error [%d] : In-valid sensor\r\n", rslt);
            break;
        case BMA4_E_CONFIG_STREAM_ERROR:
            PDEBUG("Error [%d] : Config stream error\r\n", rslt);
            break;
        case BMA4_E_SELF_TEST_FAIL:
            PDEBUG("Error [%d] : Self-test fail\r\n", rslt);
            break;
        case BMA4_E_FOC_FAIL:
            PDEBUG("Error [%d] : FOC fail\r\n", rslt);
            break;
        case BMA4_E_FAIL:
            PDEBUG("Error [%d] : E_Fail\r\n", rslt);
            break;
        case BMA4_E_INT_LINE_INVALID:
            PDEBUG("Error [%d] : Interrupt line invalid\r\n", rslt);
            break;
        case BMA4_E_RD_WR_LENGTH_INVALID:
            PDEBUG("Error [%d] : R/W length invalid\r\n", rslt);
            break;
        default:
            PDEBUG("Error [%d] : Unknown error code\r\n", rslt);
            break;
    }
}

#if defined(FIFO_POLL) || defined(FIFO_WM_INT)
uint16_t Open_BMA456H_FIFO(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	
#if defined(FIFO_WM_INT)	
	/* Variable to define two interrupt lines */
	uint8_t  int_line[2] = {BMA4_INTR1_MAP, BMA4_INTR2_MAP};
	struct bma4_int_pin_config int_pin_config;
#endif

	/* Disable the advanced power save mode to configure the FIFO buffer */
	bma4_set_advance_power_save(BMA4_DISABLE, dev);

	bma4_set_fifo_config(BMA4_FIFO_ALL, BMA4_DISABLE, dev);
	
	/* Modify the FIFO buffer instance and link to the device instance */
	bma456h_fifo_frame.data = fifo_buff;
	bma456h_fifo_frame.length = 420;
	bma456h_fifo_frame.fifo_data_enable = BMA4_ENABLE;
	bma456h_fifo_frame.fifo_header_enable = BMA4_ENABLE;
	dev->fifo = &bma456h_fifo_frame;

	bma4_set_fifo_self_wakeup(BMA4_DISABLE, dev);

	/* Configure the FIFO buffer */
	rslt = bma4_set_fifo_config((BMA4_FIFO_ACCEL | BMA4_FIFO_HEADER), BMA4_ENABLE, dev);
	if (rslt != BMA4_OK)
	{
		PDEBUG("bma4_set_fifo_config error: %d\r\n", rslt);
	}
	
	//rslt |= bma4_set_fifo_config((BMA4_FIFO_TIME), BMA4_DISABLE, dev);
	//rslt |= bma4_set_fifo_config((BMA4_FIFO_STOP_ON_FULL), BMA4_ENABLE, dev);
	//HAL_Delay(5);

#if defined(FIFO_WM_INT)
	/* Configure the FIFO watermark, 7*50 bytes = 350 bytes, for 50 Hz ODR*/
	rslt = bma4_set_fifo_wm(350, dev);
	if (rslt != BMA4_OK)
	{
		PDEBUG("bma4_set_fifo_wm error: %d\r\n", rslt);
	}
	HAL_Delay(5);
	rslt |= bma4_get_int_pin_config(&int_pin_config, int_line[1], dev);
	int_pin_config.edge_ctrl = BMA4_EDGE_TRIGGER;
	int_pin_config.lvl = BMA4_ACTIVE_HIGH;
	int_pin_config.od = BMA4_PUSH_PULL;
	int_pin_config.output_en = BMA4_OUTPUT_ENABLE;
	rslt |= bma4_set_int_pin_config(&int_pin_config, int_line[1], dev);

	/* Mapping line interrupt 1 with that of two sensor feature interrupts -
	 * Step counter and wake up interrupt */
	rslt |= bma456h_map_interrupt(int_line[1], BMA4_FIFO_WM_INT, BMA4_ENABLE, dev);
#endif

	/* Clear FIFO */
	rslt = bma4_set_command_register(0xB0, dev);
	if (rslt != BMA4_OK)
	{
		PDEBUG("bma4_set_command_register clear FIFO error: %d\r\n", rslt);
	}
	//HAL_Delay(5);
	#if defined(SUPPORT_LOWPOWER)
	rslt = bma4_set_advance_power_save(BMA4_ENABLE, dev);
	//HAL_Delay(5);
	#endif

	return rslt;
}
#endif

uint16_t Close_BMA456H_FIFO(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	
	/* Disable advanced power save */
	rslt = bma4_set_advance_power_save(BMA4_DISABLE, dev);
	if (rslt != BMA4_OK) 
	{
		PDEBUG("Error code: %d\r\n", rslt);
		return rslt;
	}

	/* Flush FIFO */
	rslt = bma4_set_command_register(0xB0, dev);
	if (rslt != BMA4_OK) 
	{
			PDEBUG("Flush FIFO error, error code: %d\r\n", rslt);
			return rslt;
	}
	rslt |= bma4_set_fifo_self_wakeup(BMA4_DISABLE, dev);
	rslt |= bma4_set_fifo_config(BMA4_FIFO_ALL, BMA4_DISABLE, dev);

	#if defined(SUPPORT_LOWPOWER)
	rslt |= bma4_set_advance_power_save(BMA4_ENABLE, dev);
	HAL_Delay(5);
	#endif

	return rslt;
}

uint16_t Open_BMA456H_STEP_COUNTER(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;

	rslt = bma456h_feature_enable(BMA456H_STEP_COUNTER_EN, TRUE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Enable BMA456H_STEP_COUNTER_EN failed\r\n");
	}
	
	rslt = bma456h_reset_step_counter(dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("bma456h_reset_step_counter failed\r\n");
	}

	return rslt;
}

uint16_t Close_BMA456H_STEP_COUNTER(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	
	rslt = bma456h_feature_enable(BMA456H_STEP_COUNTER_EN, FALSE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Disable BMA456H_STEP_COUNTER_EN failed\r\n");
	}

	return rslt;
}

uint16_t Open_BMA456H_TAP(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	struct bma456h_tap_settings settings = { 0 };
	/* Variable to define two interrupt lines */
	uint8_t int_line = BMA4_INTR1_MAP;
	struct bma4_int_pin_config int_pin_config;

	/* Enabling the single, double and triple tap features */
	rslt = bma456h_feature_enable((BMA456H_SINGLE_TAP_EN | BMA456H_DOUBLE_TAP_EN | BMA456H_TRIPLE_TAP_EN), BMA4_ENABLE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Enable TAP failed\r\n");
	}
	dev->delay(3);

	rslt = bma4_get_int_pin_config(&int_pin_config, int_line, dev);
	int_pin_config.edge_ctrl |= BMA4_LEVEL_TRIGGER;
	int_pin_config.lvl = BMA4_ACTIVE_HIGH;
	int_pin_config.od = BMA4_PUSH_PULL;
	int_pin_config.output_en = BMA4_OUTPUT_ENABLE;
	rslt = bma4_set_int_pin_config(&int_pin_config, int_line, dev);
	HAL_Delay(5);

	/* Map the interrupt 1 for tap detection */
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, BMA456H_TAP_OUT_INT, BMA4_ENABLE, dev);
	
	/* Getting tap parameters settings */
	rslt = bma456h_tap_get_parameter(&settings, dev);
	PDEBUG("Setting_8 wait_for_timeout=0x%04X, Setting_2 tap_sens_thres=0x%04X, Setting_11 max_taps=0x%04X\r\n", settings.param8, settings.param2, settings.param11);

	settings.param2 = 3;
	settings.param8 = 1;
	rslt = bma456h_tap_set_parameter(&settings, dev);

	/* Getting tap parameters settings */
	rslt = bma456h_tap_get_parameter(&settings, dev);
	PDEBUG("Setting_8 wait_for_timeout=0x%04X, Setting_2 tap_sens_thres=0x%04X, Setting_11 max_taps=0x%04X\r\n", settings.param8, settings.param2, settings.param11);

	bma4_set_interrupt_mode(BMA4_NON_LATCH_MODE, dev);
	HAL_Delay(5);
	
	return rslt;
}

uint16_t Close_BMA456H_TAP(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;

	/* Map the interrupt 1 for tap detection */
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, BMA456H_TAP_OUT_INT, BMA4_DISABLE, dev);

	if (rslt == BMA4_OK)
	{
		/* Enabling the single, double and triple tap features */
		rslt = bma456h_feature_enable((BMA456H_SINGLE_TAP_EN | BMA456H_DOUBLE_TAP_EN | BMA456H_TRIPLE_TAP_EN), BMA4_DISABLE, dev);
		dev->delay(3);
	}
	return rslt;
}

uint16_t Open_BMA456H_TILT_ON_WRIST(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;

	return rslt;
}

uint16_t Close_BMA456H_TILT_ON_WRIST(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	return rslt;
}

uint16_t Open_BMA456H_ANY_MOTION(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	struct bma456h_any_no_mot_config any_no_mot = { 0 };
	/* Variable to define two interrupt lines */
	uint8_t int_line = BMA4_INTR1_MAP;
	struct bma4_int_pin_config int_pin_config;

	rslt = bma4_get_int_pin_config(&int_pin_config, int_line, dev);
	int_pin_config.edge_ctrl |= BMA4_LEVEL_TRIGGER;
	int_pin_config.lvl = BMA4_ACTIVE_HIGH;
	int_pin_config.od = BMA4_PUSH_PULL;
	int_pin_config.output_en = BMA4_OUTPUT_ENABLE;
	rslt = bma4_set_int_pin_config(&int_pin_config, int_line, dev);
	dev->delay_ms(5);

	/* Map the interrupt 1 for any-motion */
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, (BMA456H_ANY_MOT_INT | BMA456H_NO_MOT_INT), BMA4_ENABLE, dev);

	/* Getting any motion configuration to get default configuration */
	rslt = bma456h_get_any_mot_config(&any_no_mot, dev);

	if (rslt == BMA4_OK)
	{
		/*
		* Set the slope threshold:
		*  Interrupt will be generated if the slope of all the axis exceeds the threshold (1 bit = 0.48mG)
		*/
		any_no_mot.threshold = 10;

		/*
		* Set the duration for any motion interrupt:
		*  Duration defines the number of consecutive data points for which threshold condition must be true(1
		* bit =
		* 20ms)
		*/
		any_no_mot.duration = 4;

		/* Like threshold and duration, we can also change the config of int_bhvr and slope */

		/* Set the threshold and duration configuration */
		rslt = bma456h_set_any_mot_config(&any_no_mot, dev);

		if (rslt == BMA4_OK)
		{
			/* Getting no motion configuration to get default configuration */
			rslt = bma456h_get_no_mot_config(&any_no_mot, dev);

			if (rslt == BMA4_OK)
			{
				/*
				 * Set the slope threshold:
				 *  Interrupt will be generated if the slope of all the axis exceeds the threshold (1 bit = 0.48mG)
				 */
				any_no_mot.threshold = 10;

				/*
				 * Set the duration for no motion interrupt:
				 * Duration defines the number of consecutive data points for which threshold condition must be
				 * true(1 bit = 20ms)
				 */
				any_no_mot.duration = 4;

				/* Like threshold and duration, we can also change the config of int_bhvr */

				/* Set the threshold and duration configuration */
				rslt = bma456h_set_no_mot_config(&any_no_mot, dev);
			}
		}
	}
	else
	{
		PDEBUG("bma456h_get_any_mot_config failed\r\n");
	}

	/* Enabling X, Y, and Z axis for Any and No motion feature */
	rslt = bma456h_feature_enable((BMA456H_ANY_MOTION_ALL_AXIS_EN | BMA456H_NO_MOTION_ALL_AXIS_EN), BMA4_ENABLE, dev);
	
	/* Loop over to get the interrupt status of any-motion */
	PDEBUG("Shake the sensor in any direction\r\n");
	
	return rslt;
}

uint16_t Close_BMA456H_ANY_MOTION(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;

	
	return rslt;
}

uint16_t Open_BMA456H_ORIENTATION(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	return rslt;
}

uint16_t Close_BMA456H_ORIENTATION(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	return rslt;
}

uint16_t Open_BMA456H_ACTIVITY_RECOGNITION(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	/* Variable to define two interrupt lines */
	uint8_t int_line = BMA4_INTR1_MAP;
	struct bma4_int_pin_config int_pin_config;

	/* Enabling step detector feature */
       rslt = bma456h_feature_enable(BMA456H_STEP_ACTIVITY_EN, BMA4_ENABLE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Enable STEP_ACTIVITY failed\r\n");
	}

	rslt = bma4_get_int_pin_config(&int_pin_config, int_line, dev);
	int_pin_config.edge_ctrl |= BMA4_LEVEL_TRIGGER;
	int_pin_config.lvl = BMA4_ACTIVE_HIGH;
	int_pin_config.od = BMA4_PUSH_PULL;
	int_pin_config.output_en = BMA4_OUTPUT_ENABLE;
	rslt = bma4_set_int_pin_config(&int_pin_config, int_line, dev);
	HAL_Delay(5);

	/*Map IN_EAR interrupt to INT pin 1*/
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, BMA456H_ACTIVITY_INT, BMA4_ENABLE, dev);

	bma4_set_interrupt_mode(BMA4_NON_LATCH_MODE, dev);
	HAL_Delay(5);
	
	return rslt;
}

uint16_t Close_BMA456H_ACTIVITY_RECOGNITION(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;

	/* Disabling step detector feature */
       rslt = bma456h_feature_enable(BMA456H_STEP_ACTIVITY_EN, BMA4_DISABLE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Disable STEP_ACTIVITY failed\r\n");
	}
	
	/*Unmap ACTIVITY interrupt to INT pin 1*/
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, BMA456H_ACTIVITY_INT, BMA4_DISABLE, dev);
	return rslt;
}

uint16_t Open_BMA456H_IN_OUT_EAR(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	/* Variable to define two interrupt lines */
	uint8_t int_line = BMA4_INTR1_MAP;
	struct bma4_int_pin_config int_pin_config;

	/* Enable in_ear feature */
	rslt = bma456h_feature_enable(BMA456H_IN_EAR_DETECTION_EN, BMA4_ENABLE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Enable IN_EAR_DETECTION failed\r\n");
	}

	rslt = bma4_get_int_pin_config(&int_pin_config, int_line, dev);
	int_pin_config.edge_ctrl |= BMA4_LEVEL_TRIGGER;
	int_pin_config.lvl = BMA4_ACTIVE_HIGH;
	int_pin_config.od = BMA4_PUSH_PULL;
	int_pin_config.output_en = BMA4_OUTPUT_ENABLE;
	rslt = bma4_set_int_pin_config(&int_pin_config, int_line, dev);
	HAL_Delay(5);

	/*Map IN_EAR interrupt to INT pin 1*/
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, BMA456H_IN_EAR_INT, BMA4_ENABLE, dev);

	bma4_set_interrupt_mode(BMA4_NON_LATCH_MODE, dev);
	HAL_Delay(5);
	
	return rslt;
}

uint16_t Close_BMA456H_IN_OUT_EAR(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;

	rslt = bma456h_feature_enable(BMA456H_IN_EAR_DETECTION_EN, BMA4_DISABLE, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("Disable IN_EAR_DETECTION failed\r\n");
	}

	/*Unmap IN_EAR interrupt to INT pin 1*/
	rslt = bma456h_map_interrupt(BMA4_INTR1_MAP, BMA456H_IN_EAR_INT, BMA4_DISABLE, dev);
	return rslt;
}

void BMA456H_Print_ALLRegs(struct bma4_dev *dev)
{
	uint8_t value;

	uint8_t reg_addr;

	for(reg_addr = 0; reg_addr < 0x7E; reg_addr++)
	{
		bma4_read_regs(reg_addr, &value, 1, dev);
		PDEBUG("0x%02X, value=0x%02X\r\n", reg_addr, value);
	}
}

uint16_t Init_BMA456H(struct bma4_dev *dev)
{
	uint16_t rslt = BMA4_OK;
	/* Declare an accelerometer configuration structure */
	struct bma4_accel_config accel_conf;
	uint8_t chip_id;
	
#if defined(USE_I2C_INTERFACE)
	dev->dev_addr = BMA4_I2C_ADDR_SECONDARY;
	dev->interface = BMA4_I2C_INTERFACE;
	dev->bus_read = (bma4_com_fptr_t)SensorAPI_I2Cx_Read;
	dev->bus_write = (bma4_com_fptr_t)SensorAPI_I2Cx_Write;
	dev->delay = (bma4_delay_fptr_t)DelayUs;
	dev->delay_ms = (bma4_delay_fptr_t)HAL_Delay;
	//dev->resolution = 12;
	dev->read_write_len = 32;
	//dev->feature_len = BMA456H_FEATURE_SIZE;
#elif defined(USE_SPI_INTERFACE)
	dev->dev_addr = 0x00;
	dev->interface = BMA4_SPI_INTERFACE;
	dev->bus_read = (bma4_com_fptr_t)SensorAPI_SPIx_Read;
	dev->bus_write = (bma4_com_fptr_t)SensorAPI_SPIx_Write;
	dev->delay = (bma4_delay_fptr_t)DelayUs;
	dev->delay_ms = (bma4_delay_fptr_t)HAL_Delay;
	//dev->resolution = 12;
	//dev->feature_len = BMA456H_FEATURE_SIZE;
	dev->read_write_len  = 32;
#endif

	bma4_read_regs(BMA4_CHIP_ID_ADDR, &chip_id, 1, dev);
	/* Initialize the instance */
	rslt = bma456h_init(dev);

	PDEBUG("Chip ID 0x%x\r\n", dev->chip_id);
	
	/* BMA4xy softreset */
	rslt = bma4_set_command_register(0xB6, dev);
	//vTaskDelay(200);
	dev->delay_ms(200);

	rslt = bma4_read_regs(BMA4_CHIP_ID_ADDR, &chip_id, 1, dev);

	/* Load configuration file */
	rslt = bma456h_write_config_file(dev);
	if (rslt != BMA4_OK) 
	{
	    PDEBUG("Load configuration fail\r\n");
	} 
	else 
	{
	    PDEBUG("Load configuration successful\r\n");
	}
	//vTaskDelay(5);
	dev->delay_ms(5);

	/* Enable the accelerometer */
	bma4_set_accel_enable(BMA4_ENABLE, dev);
	//vTaskDelay(5);
	dev->delay_ms(5);

#if defined(SELF_TEST)
	uint8_t result;
	/* Doing soft reset */
	bma4_soft_reset(dev);
	print_bma456_rslt(rslt);
	PDEBUG("Running self test...\r\n");
	vTaskDelay(500);
	
	bma4_perform_accel_selftest(&result, dev);
	print_bma456_rslt(rslt);

	if (result == BMA4_SELFTEST_PASS) 
	{
		PDEBUG("Self test passed!\r\n");
	}
	else
	{
		PDEBUG("Self test failed!\r\n");
	}
	while(1);
#endif

#if defined(TAP)
	/* Assign the desired settings */
	accel_conf.odr = BMA4_OUTPUT_DATA_RATE_200HZ;
#else
	/* Assign the desired settings */
	accel_conf.odr = BMA4_OUTPUT_DATA_RATE_50HZ;
#endif
	accel_conf.range = BMA4_ACCEL_RANGE_4G;
#if defined(SUPPORT_LOWPOWER)
	accel_conf.bandwidth = BMA4_ACCEL_OSR4_AVG1;
	accel_conf.perf_mode = BMA4_CIC_AVG_MODE;
#else
	accel_conf.bandwidth = BMA4_ACCEL_NORMAL_AVG4;
	accel_conf.perf_mode = BMA4_CONTINUOUS_MODE;
#endif

	/* Set the configuration */
	rslt = bma4_set_accel_config(&accel_conf, dev);
	if(rslt != BMA4_OK)
	{
		PDEBUG("bma4_set_accel_config failed\r\n");
	}

	#if defined(STEP_COUNTER)
	Open_BMA456H_STEP_COUNTER(dev);
	#endif
	
	#if defined(TAP)
	Open_BMA456H_TAP(dev);
	#endif
	
	#if defined(ANY_NO_MOTION)
	Open_BMA456H_ANY_MOTION(dev);
	#endif
	
	#if defined(ORIENTATION)
	#endif
	
	#if defined(ACTIVITY_RECOGNITION)
	Open_BMA456H_ACTIVITY_RECOGNITION(dev);
	#endif
	
	#if defined(IN_OUT_EAR)
	Open_BMA456H_IN_OUT_EAR(dev);
	#endif

	#if defined(ANY_NO_MOTION)
	Open_BMA456H_ANY_MOTION(dev);
	#endif

	#if defined(FIFO_POLL) || defined(FIFO_WM_INT)
	Open_BMA456H_FIFO(dev);
	#endif

	#if defined(TAP) || defined(IN_OUT_EAR) || defined(ANY_NO_MOTION)
	Enable_MCU_INT1_Pin();
	#endif

	#if defined(FIFO_WM_INT)
	Enable_MCU_INT2_Pin();
	#endif

	return rslt;
}

#if defined(FIFO_POLL) || defined(STEP_COUNTER)
void BMA456H_Timer_Callback(TimerHandle_t xTimer)
{
	bma456h_fifo_ready = 1;
}
#endif

#if defined(TAP) || defined(IN_OUT_EAR) || defined(ANY_MOTION)
void StartBMA456InterruptTask(void const * argument)
{
	int8_t rslt = BMA4_OK;
	struct bma4_dev *dev;
	struct bma456h_out_state tap_out = { 0 };
	struct bma456h_out_state ear_detect_out = { 0 };
	struct bma456h_out_state activity_output = { 0 };

	dev = &bma456accel;

	/* Variable to get wrist gesture status */
	uint16_t int_status = 0;

	for(;;)
	{
		if(int1_flag == 1)
		{
			int_status = 0;
			/* Read interrupt status */
			rslt = bma456h_read_int_status(&int_status, dev);

			/* Filtering only the tap interrupt */
			if (rslt == BMA4_OK)
			{
				if(int_status & BMA456H_TAP_OUT_INT)
				{
					rslt = bma456h_output_state(&tap_out, dev);
					if (tap_out.single_tap)
					{
						PDEBUG("Single Tap interrupt occurred\r\n");
					}
					else if (tap_out.double_tap)
					{
						PDEBUG("Double Tap interrupt occurrer\n");
					}
					else if (tap_out.triple_tap)
					{
						PDEBUG("Triple Tap interrupt occurred\r\n");
					}
				}
				/* Enters only if the obtained interrupt is in-ear */
				if(int_status & BMA456H_IN_EAR_INT)
				{
					PDEBUG("In_ear interrupt occurred\r\n");

					/* Read the status of the hearing device */
					rslt = bma456h_output_state(&ear_detect_out, dev);

					if (rslt == BMA4_OK)
					{
						PDEBUG("The ear detection output is %d\r\n", ear_detect_out.ear_detection_out);

						switch (ear_detect_out.ear_detection_out)
						{
							case BMA456H_DEVICE_UNKNOWN:
								PDEBUG("\nDevice is in unknown state\r\n");
								break;
							case BMA456H_DEVICE_NOT_IN_EAR:
								PDEBUG("\nDevice is not in ear\r\n");
								break;
							case BMA456H_DEVICE_WEARING:
								PDEBUG("\nDevice is wearing\r\n");
								break;
							case BMA456H_DEVICE_REMOVING:
								PDEBUG("\n Device is removing\r\n");
								break;
							case BMA456H_DEVICE_IN_EAR:
								PDEBUG("\n Device is in ear\r\n");
								break;
							default:
								break;
						}
					}
				}
				/* Enters only if the obtained interrupt is any-motion */
				if (int_status & BMA456H_ANY_MOT_INT)
				{
					PDEBUG("Any-motion interrupt occurred\r\n");
				}
				/* Enters only if the obtained interrupt is no-motion */
				if (int_status & BMA456H_NO_MOT_INT)
				{
					PDEBUG("No-motion interrupt occurred\r\n");
				}
				/* Filtering only the activity interrupt */
				if(int_status & BMA456H_ACTIVITY_INT)
				{
					PDEBUG("Activity interrupt occurred\r\n");

					/* Get step activity output */
					rslt = bma456h_output_state(&activity_output, dev);
					PDEBUG("The Activity output is %d\r\n", activity_output.activity_type);

					switch (activity_output.activity_type)
					{
						case BMA456H_USER_STATIONARY:
							PDEBUG("User state is stationary\r\n");
							break;
						case BMA456H_USER_WALKING:
							PDEBUG("User state is walking\r\n");
							break;
						case BMA456H_USER_RUNNING:
							PDEBUG("User state is running\r\n");
							break;
						case BMA456H_UNKNOWN_ACTVTY:
							PDEBUG("User state is invalid state\r\n");
							break;
						default:
							break;
					}
				}
			}
			int1_flag = 0;
		}
	}
}
#endif

void StartBMA456HTask(void const * argument)
{
	uint16_t rslt = BMA4_OK;
	
	#if defined(DATA_POLL) || defined(DATA_POLL)
	/* Declare an instance of the sensor data structure */
	struct bma4_accel sensor_raw_data;
	#endif

	struct bma4_dev *dev;
	dev = &bma456accel;

	#if defined(TAP) || defined(IN_OUT_EAR) || defined(ANY_MOTION)
	osThreadId BMA456InterruptTaskHandle;
	osThreadDef(BMA456InterruptTask, StartBMA456InterruptTask, osPriorityNormal, 0, 256);
  	BMA456InterruptTaskHandle = osThreadCreate(osThread(BMA456InterruptTask), NULL);
#endif

	Init_BMA456H(dev);
	//BMA456H_Print_ALLRegs(dev);

	#if defined(FIFO_POLL) || defined(STEP_COUNTER)
	BMA456H_Timer_Handler = xTimerCreate("BMA456HFIFOReadTimer", 1020, pdTRUE, (void *)1, BMA456H_Timer_Callback);
	if(BMA456H_Timer_Handler != NULL)
	{
		xTimerStart(BMA456H_Timer_Handler, 1000);
	}
	#endif

	for(;;)
	{
	#if defined(DATA_POLL)
		//PDEBUG("----------------------Data Poll Test Case----------------------\r\n");
		vTaskDelay(500);
		rslt = bma4_read_accel_xyz(&sensor_raw_data, dev);
		if (rslt != BMA4_OK) 
		{
			PDEBUG("Read accel xyz error: %d\r\n", rslt);
		}

		/* Use the data */
		PDEBUG("Poll data X: %d, Y: %d, Z: %d\r\n", sensor_raw_data.x, sensor_raw_data.y, sensor_raw_data.z);
	#elif defined(FIFO_POLL) || defined(FIFO_WM_INT)
		if(bma456h_fifo_ready == 1)
		{
			#if defined(FIFO_POLL) || defined(FIFO_WM_INT)
			bma4_set_advance_power_save(BMA4_DISABLE, dev);
			HAL_Delay(5);
			
			PDEBUG("---- Read FIFO data ----\r\n");
			#if 0
			rslt = bma4_get_fifo_length(&bma456h_fifo_length, dev);
			if(rslt != BMA4_OK)
			{
				PDEBUG("bma4_get_fifo_length error:%d\r\n", rslt);
			}
			else
			{
				PDEBUG("Get fifo length=%d\r\n", bma456h_fifo_length);
			}
			#endif

			//if(fifo_length > 0)
			{
				/* Read data from the sensor FIFO buffer */
				rslt = bma4_read_fifo_data(dev); // Read FIFO data

				/* Exit the program in case of a failure */
				if (rslt != BMA4_OK)
				{
					PDEBUG("Get fifo data failed\r\n");
				}
				else
				{
					/* Parse the FIFO until there are less frames than requested */
					/* Parse the FIFO buffer and extract required number of accelerometer data frames */
					rslt = bma4_extract_accel(bma456h_sensor_data, &bma456h_acc_frames, dev);
					if(rslt != BMA4_OK)
					{
						PDEBUG("bma4_extract_accel failed error:%d\r\n", rslt);
					}
					else
					{
						PDEBUG("Parsed accelerometer data frames: %d\r\n", bma456h_acc_frames);
					}	
					    
					/* Use the accelerometer data frames */
					for (uint16_t i = 0; i < bma456h_acc_frames; i++)
					{
						PDEBUG("Index %d, X:%d, Y:%d, Z:%d\r\n", i, bma456h_sensor_data[i].x, bma456h_sensor_data[i].y, bma456h_sensor_data[i].z);
					}
				}
			}
			#endif
			#if defined(STEP_COUNTER)
			/* On interrupt, Get step counter output */
                    rslt = bma456h_step_counter_output(&bma456step_cnt, dev);
                    if (rslt == BMA4_OK) {
                        PDEBUG("The step counter output is %u\r\n", bma456step_cnt);                
                    } 
			#endif
			#if defined(SUPPORT_LOWPOWER)
			bma4_set_advance_power_save(BMA4_ENABLE, dev);
			#endif
			bma456h_fifo_ready = 0;
		}
	#endif	
	#if defined(STEP_COUNTER)
		rslt = bma456h_step_counter_output(&bma456step_cnt, dev);
		if (rslt == BMA4_OK)
		{
		    PDEBUG("The step counter output is %u\r\n", bma456step_cnt);                
		} 
		else
		{
			PDEBUG("bma456h_step_counter_output error:=%d\r\n", rslt);
		}
		vTaskDelay(1000);
	#endif
	}
}

#endif

